// -*- C++ -*-
/*!
 * @file  MapImageSelect.h
 * @brief 地図データDBからタイル画像を取得し地図を作成する
 * @date  $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * New BSD
 *
 * $Id$
 */

#ifndef MAPIMAGESELECT_H
#define MAPIMAGESELECT_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "MapImageSelectorSVC_impl.h"
#include "local_basic_type.h"
#include "VersatileOverheadSystem.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "MapDataDBControlStub.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

/*====================================================
	マクロ定義
===================================================*/
//マトリクスモード
enum MATRIX_MODE_NUM{
	MODE_DISP_OFF = 0,					//画像非表示
	MODE_CREATING,						//画像作成中
	MODE_DISP_ON,						//画像表示中
	MODE_MAX,
};
#define MODE_NO_CHG		0xFFFF			//モード切替なし

//マトリクストリガ
enum MATRIX_TRIGGER_NUM{
	TRG_MAPDB_GET_MAPINFO_SUCCESS = 0,	//地図画像付加情報取得正常終了
	TRG_MAPDB_GET_MAPINFO_ERROR,		//地図画像付加情報取得異常終了
	TRG_MAPDB_GET_MAPDATA_SUCCESS,		//地図画像付加データ取得正常終了
	TRG_MAPDB_GET_MAPDATA_ERROR,		//地図画像付加データ取得異常終了
	TRG_REQ_MAP_IMAGE_SELECT,			//地図画像セレクト要求
	TRG_REQ_SCROLL,						//スクロール要求
	TRG_REQ_ADJ_ZOOMLEVEL,				//ズームレベル切り替え要求
	TRG_REQ_MAP_CHENGE,					//地図切り替え要求
	TRG_MAX,
};

#define TRG_NON			0xFFFF			//トリガなし

//マトリクスアクション
enum MATRIX_ACT_NUM{
	ACT_MAP_CREATE_START = 0,			//地図作成開始
	ACT_GET_MAPINFO,					//画像付加情報取得
	ACT_GET_MAPDATA,					//画像付加データ取得
	ACT_MAP_CREATE_ERROR,				//地図作成失敗
	ACT_REQ_SCROLL,						//スクロール要求
	ACT_REQ_ZOOM_LEVEL,					//ズームレベル調整ようきゅ
	ACT_REQ_MAP_CHG,					//地図切り替え要求
	ACT_REQ_NOTICE_SUCCESS,				//正常終了通知
	ACT_REQ_NOTICE_ERROR,				//異常終了通知
	ACT_MAX,
};
#define ACT_NON		0xFFFF				//アクションなし

//地図画像関連
#define	TILE_X_MAXNUM			8		//X軸方向に表示可能なタイル最大数
#define	TILE_Y_MAXNUM			8		//Y軸方向に表示可能なタイル最大数
#define	TILE_MAX_NUM			64		//地図画像に使用可能なタイルの総数

//フォルダパス置換
#define FOLDER_PATH_DRONE	"AerialPhotograph"		//会津大学撮影ドローン画像の格納パス
#define FOLDER_PATH_BASEMAP	"std"		//国土地理院の基準地図格納パス


#define	CREATE_KIND_SELECT			0	//地図セレクト
#define	CREATE_KIND_SCROLL			1	//スクロール
#define	CREATE_KIND_ZOOM			2	//ズームレベル調整
#define	CREATE_KIND_MAP_CHENGE		3	//地図切り替え
#define	CREATE_KIND_MAX				4

/***************************************************************
	struct定義
***************************************************************/
struct AllMapDBAddData_st{
	MapDBAddData_st stTile[TILE_MAX_NUM];
};

/*!
 * @class MapImageSelect
 * @brief 地図データDBからタイル画像を取得し地図を作成する
 *
 * 別紙"機能仕様書_地図画像セレクトRTC.docx"参照
 *
 * 別紙"機能仕様書_地図画像セレクトRTC.docx"参照
 *
 * 別紙"機能仕様書_地図画像セレクトRTC.docx"参照
 *
 * 別紙"機能仕様書_地図画像セレクトRTC.docx"参照
 *
 */
class MapImageSelect
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  MapImageSelect(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~MapImageSelect();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * 地図画像セレクトRTCの初期化処理
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * 地図画像セレクトRTCの終了処理
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * 地図画像セレクトRTCの開始処理
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * 地図画像セレクトRTCの停止処理
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * 地図画像セレクトRTCの周期処理
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   * 地図画像セレクトRTCのエラー遷移処理
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::TimedString m_inMapAdditinalDataInfo;
  /*!
   * 地図画像付加情報をjyson形式で出力
   * - Type: RTC::TimedString
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 地図画像生成時(地図画像付加情報の取得要求を行った
   *              時)
   * - Operation Cycle: 地図画像生成時(地図画像付加情報の取得要求を
   *                    行った時)
   */
  InPort<RTC::TimedString> m_inMapAdditinalDataInfoIn;
  RTC::TimedString m_inMapAdditinalData;
  /*!
   * 地図画像付加データをjyson形式で出力
   * - Type: RTC::TimedString
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 地図画像生成時(地図画像付加データの取得要求を行っ
   *              た時)
   * - Operation Cycle: 地図画像生成時(地図画像付加データの取得要求
   *                    を行った時)
   */
  InPort<RTC::TimedString> m_inMapAdditinalDataIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::GPSData m_outMapImageStartGpsData;
  /*!
   * 地図画像の起点座標を提供するデータポート
   * - Type: RTC::GPSData
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 地図画像生成時(地図作成要求または、スクロール要求
   *              受信時)
   * - Operation Cycle: 地図画像生成時(地図作成要求または、スクロー
   *                    ル要求受信時)
   */
  OutPort<RTC::GPSData> m_outMapImageStartGpsDataOut;
  RTC::GPSData m_outMapImageEndGpsData;
  /*!
   * 地図画像の終点座標を提供するデータポート
   * - Type: RTC::GPSData
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 地図画像生成時(地図作成要求または、スクロール要求
   *              受信時)
   * - Operation Cycle: 地図画像生成時(地図作成要求または、スクロー
   *                    ル要求受信時)
   */
  OutPort<RTC::GPSData> m_outMapImageEndGpsDataOut;
  RTC::CameraImage m_outMapImageData;
  /*!
   * タイル画像から作成した地図画像を提供するデータポート
   * - Type: RTC::CameraImage
   * - Number: 1
   * - Semantics: -
   * - Unit: -
   * - Frequency: 地図画像生成時(地図作成要求または、スクロール要求
   *              受信時)
   * - Operation Cycle: 地図画像生成時(地図作成要求または、スクロー
   *                    ル要求受信時)
   */
  OutPort<RTC::CameraImage> m_outMapImageDataOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   * 地図画像セレクトRTCへのリクエスト関連のインターフェース
   * Interface: 地図画像選択要求や、スクロール要求などのインターフ
   *            ェースを提供する
   */
  RTC::CorbaPort m_MapImageSelectReqServPort;
  /*!
   * 地図画像セレクトRTCのステータス関連のインターフェース
   * Interface: 動作完了通知などのインターフェースを提供する
   */
  RTC::CorbaPort m_MapImageStsServPort;
  /*!
   * 地図データDB RTCへのリクエスト関連のインターフェース
   * Interface: 地図画像付加データ取得要求などのインターフェースを
   *            提供する
   */
  RTC::CorbaPort m_MapDataDBReqServPort;
  /*!
   * 地図データDB RTCのステータス関連のインターフェース
   * Interface: 動作完了通知などのインターフェースを提供する
   */
  RTC::CorbaPort m_MapDataDBStsServPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  /*!
   * 合成計画リストから選択したタイル画像を元に地図画像を生成するた
   * めの要求を行う
   * - Argument:      別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Return Value:  別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  MapImageReqSelectImageSVC_impl m_MapImageReqSelectImageProv;
  /*!
   * 地図画像のスクロール要求を行う
   * - Argument:      別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Return Value:  別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  MapImageReqScrollSVC_impl m_MapImageReqScrollProv;
  /*!
   * 地図画像のズームレベル設定要求を行う
   * - Argument:      別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Return Value:  別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  MapImageReqAdjZoomSVC_impl m_MapImageReqAdjZoomProv;
  /*!
   * 地図切り替え要求を行う
   * - Argument:      別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Return Value:  別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  MapImageReqAdjMapChangeSVC_impl m_MapImageReqAdjMapChangeProv;
  /*!
   * 地図画像セレクトRTCの動作完了通知を行う
   * - Argument:      別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Return Value:  別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  MapImageNoticeSVC_impl m_MapImageNoticeProv;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   * 地図画像付加データ情報の取得要求を行う
   * - Argument:      別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Return Value:  別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  RTC::CorbaConsumer<MapDataDBReqServ::MapDBReqGetAddDataInfo> m_MapDBReqGetAddDataInfoCons;
  /*!
   * 地図画像付加データの取得要求を行う
   * - Argument:      別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Return Value:  別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  RTC::CorbaConsumer<MapDataDBReqServ::MapDBReqGetAddData> m_MapDBReqGetAddDataCons;
  /*!
   * 地図データDB RTCの動作完了通知を行う
   * - Argument:      別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Return Value:  別紙"機能仕様書_地図画像セレクトRTC.docx"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  RTC::CorbaConsumer<MapDataDBStsServ::MapDataDBNotice> m_MapDataDBNoticeCons;
  
  // </rtc-template>

 private:
	 bool_type blMapImgSelMatrixTrigerSet(void);
	 uhword_type uhwMapImgSelMatrixCheck(void);
	 uhword_type uhwMapImgSelProcExe(void);
	 bool_type blMapDBNoticeTrigCheck(uhword_type &);
	 bool_type blReqSelectImageTrigCheck(uhword_type &);
	 bool_type blReqScrollTrigCheck(uhword_type &);
	 bool_type blReqAdjZoomTrigCheck(uhword_type &);
	 bool_type blReqAdjMapChgTrigCheck(uhword_type &);
	 uhword_type uhwProcReqSelectImage(void);
	 uhword_type uhwProcGetMapDBAddInfo(void);
	 uhword_type uhwProcGetMapDBAddData(void);
	 uhword_type uhwProcMapDBError(void);
	 uhword_type uhwProcScroll(void);
	 uhword_type uhwProcAdjZoom(void);
	 uhword_type uhwProcAdjMapChg(void);
	 uhword_type uhwProcReqNoticeSuccess(void);
	 uhword_type uhwProcReqNoticeError(void);
	 uhword_type uhwMapDBReqGetAddData(uword_type,uword_type);
	 uhword_type uhwGetStartTileData(void);
	 uhword_type uhwGetNextTileData(void);
	 uhword_type uhwCreateMapImage(void);
	 uhword_type uhwGetGpsData(void);
	 void MapImageNotice(uhword_type);
	 bool_type blMapDBDBReqGetAddDataInfoConsChk(void);
	 bool_type blMapDBReqGetAddDataConsChk(void);
	 bool_type blMapDataDBNoticeConsChk(void);
	 uhword_type uhwMapDataRangeCheck(MapDBAddData_st);
	 std::string strUTF8toSjis(std::string);
	 std::string strFolderPathReplace(std::string,uword_type);
	 void MapImgSelInit(void);
	 void MapAddDataInit(void);
	 uhword_type uhwZoomLevelAdjCheck(void);
	 std::string strMapJsonDataReplace(std::string);
	 void MapImgSelModeSet(uhword_type);

	//マトリクス制御用
	 uhword_type m_uhwMatrixTriger;			//マトリクス内部トリガ
	 uhword_type m_uhwMatrixMode;			//マトリクスモード
	 uhword_type m_uhwMatrixActID;			//マトリクスアクションID
  
	//入力となる完了通知、要求のバックアップ用
	 uword_type	m_uwMapDBNotice;			//地図データDB完了通知
	 uword_type	m_uwSelImageGroupId;		//画像選択要求時のグループID
	 uword_type	m_uwScrollArrow;			//十字キー押下方向
	 uword_type m_uwAdjZoom;				//ズームレベル調整
	 uword_type	m_uwZoomLevel;				//現在のズームレベル設定値
	 uword_type	m_uwMapKind;				//地図種別

	//地図画像付加データ保存用
	 MapDBAddInfo_st m_stMapDBAddInfo;		//地図画像付加情報
	 AllMapDBAddData_st m_stMapDBAddData;	//地図画像付加データ
	 uword_type m_uwMapDBGetCnt;			//地図画像付加データ取得回数
	 uword_type	m_uwTileNoXStart;			//現在表示で使用しているX軸のタイルNo（始点）
	 uword_type m_uwTileNoYStart;			//現在表示で使用しているY軸のタイルNo（始点）
	 uword_type	m_uwTileNoXEnd;				//現在表示で使用しているX軸のタイルNo（終点）
	 uword_type m_uwTileNoYEnd;				//現在表示で使用しているY軸のタイルNo（終点）
	 uword_type m_uwMapCreateKind;			//地図作成種別
	 uword_type m_uwTileXGetCnt;
	 uword_type m_uwTileYGetCnt;
};


extern "C"
{
  DLL_EXPORT void MapImageSelectInit(RTC::Manager* manager);
};

#endif // MAPIMAGESELECT_H
