// -*-C++-*-
/*!
 * @file  MapImageSelectorSVC_impl.cpp
 * @brief Service implementation code of MapImageSelector.idl
 *
 * @author 株式会社 東日本計算センター
 *
 * New BSD
 *
 */

#include "MapImageSelectorSVC_impl.h"
#include "VersatileOverheadSystem.h"
#include <iostream>

/*
 * Example implementational code for IDL interface MapImageSelectReqServ::MapImageReqSelectImage
 */
MapImageReqSelectImageSVC_impl::MapImageReqSelectImageSVC_impl()
{
	m_ulGroupId = GROUP_ID_NON;
}


MapImageReqSelectImageSVC_impl::~MapImageReqSelectImageSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::UShort MapImageReqSelectImageSVC_impl::usSetMapImageReqSelectImage(CORBA::ULong ulGroupId)
{
	if(ulGroupId != GROUP_ID_NON){
		m_ulGroupId = ulGroupId;
	}
	else{
		std::cout << "[error]MapImageReqSelectImageSVC_impl::usSetMapImageReqSelectImage():ParamErr = " << ulGroupId << std::endl;
		return RESULT_NACK;
	}

	return RESULT_ACK;
}

void MapImageReqSelectImageSVC_impl::GetMapImageReqSelectImage(CORBA::ULong& ulGroupId)
{
	if(&ulGroupId != NULL){
		ulGroupId = m_ulGroupId;
	}
	else{
		std::cout << "[error]MapImageReqSelectImageSVC_impl::GetMapImageReqSelectImage() = " << ulGroupId << std::endl;
	}
}

void MapImageReqSelectImageSVC_impl::InitMapImageReqSelectImage(void)
{
	m_ulGroupId = GROUP_ID_NON;
}


// End of example implementational code

/*
 * Example implementational code for IDL interface MapImageSelectReqServ::MapImageReqScroll
 */
MapImageReqScrollSVC_impl::MapImageReqScrollSVC_impl()
{
	m_ulArrow = ARROW_DEF;
}


MapImageReqScrollSVC_impl::~MapImageReqScrollSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::UShort MapImageReqScrollSVC_impl::usSetMapImageReqScroll(CORBA::ULong ulArrow)
{
	if(ulArrow < ARROW_MAX){
		m_ulArrow = ulArrow;
	}
	else{
		std::cout << "[error]MapImageReqSelectImageSVC_impl::usSetMapImageReqScroll():ParamErr = " << ulArrow << std::endl;
		return RESULT_NACK;
	}
	return RESULT_ACK;
}

void MapImageReqScrollSVC_impl::GetMapImageReqScroll(CORBA::ULong& ulArrow)
{
	if(&ulArrow != NULL){
		ulArrow = m_ulArrow;
	}
	else{
		std::cout << "[error]MapImageReqSelectImageSVC_impl::GetMapImageReqScroll() = " << ulArrow << std::endl;
	}
}

void MapImageReqScrollSVC_impl::InitMapImageReqScroll(void)
{
	m_ulArrow = ARROW_DEF;
}

// End of example implementational code

/*
 * Example implementational code for IDL interface MapImageSelectReqServ::MapImageReqAdjZoom
 */
MapImageReqAdjZoomSVC_impl::MapImageReqAdjZoomSVC_impl()
{
	m_ulZoomAdj = ZOOM_ADJ_OFF;
}


MapImageReqAdjZoomSVC_impl::~MapImageReqAdjZoomSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::UShort MapImageReqAdjZoomSVC_impl::usSetMapImageReqAdjZoom(CORBA::ULong ulZoomAdj)
{
	if(ulZoomAdj < ZOOM_ADJ_MAX){
		m_ulZoomAdj = ulZoomAdj;
	}
	else{
		std::cout << "[error]MapImageReqSelectImageSVC_impl::usSetMapImageReqAdjZoom():ParamErr = " << ulZoomAdj << std::endl;
		return RESULT_NACK;
	}

	return RESULT_ACK;
}

void MapImageReqAdjZoomSVC_impl::GetMapImageReqAdjZoom(CORBA::ULong& ulZoomAdj)
{
	if(&ulZoomAdj != NULL){
		ulZoomAdj = m_ulZoomAdj;
	}
	else{
		std::cout << "[error]MapImageReqSelectImageSVC_impl::GetMapImageReqAdjZoom()" << std::endl;
	}
}

void MapImageReqAdjZoomSVC_impl::InitMapImageReqAdjZoom(void)
{
	m_ulZoomAdj = ZOOM_ADJ_OFF;
}

// End of example implementational code

/*
 * Example implementational code for IDL interface MapImageSelectReqServ::MapImageReqAdjMapChange
 */
MapImageReqAdjMapChangeSVC_impl::MapImageReqAdjMapChangeSVC_impl()
{
	m_ulMapKind = MAP_CHENGE_DEF;
}


MapImageReqAdjMapChangeSVC_impl::~MapImageReqAdjMapChangeSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::UShort MapImageReqAdjMapChangeSVC_impl::usSetMapImageReqAdjMapChange(CORBA::ULong ulMapKind)
{
	if(ulMapKind < MAP_KIND_MAX){
		m_ulMapKind = ulMapKind;
	}
	else{
		std::cout << "[error]MapImageReqSelectImageSVC_impl::usSetMapImageReqAdjMapChange():ParamErr = " << ulMapKind << std::endl;
		return RESULT_NACK;
	}

	return RESULT_ACK;
}

void MapImageReqAdjMapChangeSVC_impl::GetMapImageReqAdjMapChange(CORBA::ULong& ulMapKind)
{
	if(&ulMapKind != NULL){
		ulMapKind = m_ulMapKind;
	}
	else{
		std::cout << "[error]MapImageReqSelectImageSVC_impl::GetMapImageReqAdjMapChange() = " << ulMapKind << std::endl;
	}
}

void MapImageReqAdjMapChangeSVC_impl::InitMapImageReqAdjMapChange(void)
{
	m_ulMapKind = MAP_CHENGE_DEF;
}

// End of example implementational code

/*
 * Example implementational code for IDL interface MapImageStsServ::MapImageNotice
 */
MapImageNoticeSVC_impl::MapImageNoticeSVC_impl()
{
	m_ulNotice = NOTICE_INIT;
	m_ulParam = NOTICE_PARAM_INIT;
}


MapImageNoticeSVC_impl::~MapImageNoticeSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::UShort MapImageNoticeSVC_impl::usSetMapImageNotice(CORBA::ULong ulNotice, CORBA::ULong ulPram)
{
	m_ulNotice = ulNotice;
	m_ulParam = ulPram;
	return RESULT_ACK;
}

void MapImageNoticeSVC_impl::GetMapImageNotice(CORBA::ULong& ulNotice, CORBA::ULong& ulPram)
{
	if((&ulNotice != NULL) && (&ulPram != NULL)){
		ulNotice = m_ulNotice;
		ulPram = m_ulParam;
	}
	else{
		std::cout << "[error]MapImageReqSelectImageSVC_impl::GetMapImageNotice() = " << ulNotice << " ," << ulPram << std::endl;
	}
}

void MapImageNoticeSVC_impl::InitMapImageNotice(void)
{
	m_ulNotice = NOTICE_INIT;
	m_ulParam = NOTICE_PARAM_INIT;
}

// End of example implementational code



